/*
 * Copyright 2020 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.gaellalire.vestige.jvm_enhancer.boot;

import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodHandles.Lookup;
import java.lang.invoke.VarHandle;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.concurrent.Callable;

/**
 * @author Gael Lalire
 */
public final class FinalUnsetter {

    private FinalUnsetter() {
    }

    public static <E> E unsetFinalField(final Field field, final Callable<E> callable) throws Exception {
        Lookup privateLookupIn = MethodHandles.privateLookupIn(Field.class, MethodHandles.lookup());
        VarHandle findVarHandle = privateLookupIn.findVarHandle(Field.class, "modifiers", int.class);
        int modifierValue = ((Integer) findVarHandle.get(field)).intValue();
        findVarHandle.set(field, modifierValue & ~Modifier.FINAL);
        try {
            return callable.call();
        } finally {
            findVarHandle.set(field, modifierValue);
        }
    }

}
