/*
 * Copyright 2020 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.gaellalire.vestige.jvm_enhancer.runtime;

import java.lang.reflect.Field;
import java.util.HashMap;

import fr.gaellalire.vestige.core.StackedHandler;

/**
 * @author Gael Lalire
 */
public class LoaderHandlerLoaderTable extends HashMap<Object, Object> implements StackedHandler<HashMap<Object, Object>> {

    private static final long serialVersionUID = 6740958307562478165L;

    private static Class<?> loaderHandlerKeyClass;

    private static Field loaderHandlerKeyParentField;

    private HashMap<Object, Object> nextHandler;

    static {
        try {
            loaderHandlerKeyClass = Class.forName("sun.rmi.server.LoaderHandler$LoaderKey");
            loaderHandlerKeyParentField = loaderHandlerKeyClass.getDeclaredField("parent");
            loaderHandlerKeyParentField.setAccessible(true);
        } catch (Exception e) {
            // keep classloader leak
            loaderHandlerKeyClass = null;
        }
    }

    public Object toKey(final Object key) {
        if (loaderHandlerKeyClass == null) {
            return key;
        }
        ClassLoader classLoader;
        try {
            classLoader = (ClassLoader) loaderHandlerKeyParentField.get(key);
            loaderHandlerKeyParentField.set(key, null);
        } catch (Exception e) {
            return key;
        }
        return new LoaderHandlerLoaderKey(classLoader, key);
    }

    @Override
    public Object get(final Object key) {
        return super.get(toKey(key));
    }

    @Override
    public Object put(final Object key, final Object value) {
        return super.put(toKey(key), value);
    }

    @Override
    public Object remove(final Object key) {
        return super.remove(toKey(key));
    }

    @Override
    public HashMap<Object, Object> getNextHandler() {
        return nextHandler;
    }

    @Override
    public void setNextHandler(final HashMap<Object, Object> nextHandler) {
        this.nextHandler = nextHandler;
    }

}
