/*
 * Copyright 2020 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.gaellalire.vestige.jvm_enhancer.runtime;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

/**
 * @author Gael Lalire
 * @param <K> key type
 * @param <V> value type
 */
public class WeakHashtable<K, V> extends Hashtable<K, V> {

    private static final long serialVersionUID = -7590280733376745723L;

    private Map<K, WeakReference<V>> weakMap = new HashMap<K, WeakReference<V>>();

    @Override
    public synchronized V get(final Object key) {
        WeakReference<V> weakReference = weakMap.get(key);
        if (weakReference == null) {
            return null;
        }
        return weakReference.get();
    }

    @Override
    public synchronized V put(final K key, final V value) {
        WeakReference<V> old = weakMap.put(key, new WeakReference<V>(value));
        if (old == null) {
            return null;
        }
        return old.get();
    }

    @Override
    public synchronized void clear() {
        weakMap.clear();
    }

}
